import 'package:adpluginx/Models/Ads/Facebook/FacebookIdsModel.dart';
import 'package:adpluginx/Models/Ads/Google/GoogleIdsModel.dart';
import 'package:adpluginx/Models/Ads/Ids/AdIds.dart';
import 'package:adpluginx/Models/Ads/IronSource/IronSourceIdsModel.dart';
import 'package:adpluginx/Models/Ads/Unity/UnityIdsModel.dart';
import 'package:adpluginx/Models/MainModel.dart';
import 'package:adpluginx/Models/VersionSpecific/ConfigModel.dart';
import 'package:adpluginx/utils/LoggerX.dart';
import 'package:adpluginx_example/Screens/MainScreen.dart';
import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import '../utils/App.dart';

class SplashScreen extends StatefulWidget {
  static const route = "/Splash";

  const SplashScreen({Key? key}) : super(key: key);

  @override
  State<SplashScreen> createState() => _SplashScreenState();
}

class _SplashScreenState extends State<SplashScreen> {
  @override
  void initState() {
    Future.microtask(() => fetchMainJson());
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return const Center(
      child: FlutterLogo(),
    );
  }

  fetchMainJson() async {
    Dio().get("http://miracocopepsi.com/test.json").then((value) {
      Response response = value;
      LoggerX.printSuccess("Main Json Fetched Completed");
      App.adPluginX.initAdPluginX(
        context,
        MainModel(
          adIds: AdIds(
              google: GoogleIdsModel.fromJson(response.data['adIds']['google']),
              unity: UnityIdsModel.fromJson(response.data['adIds']['unity']),
              ironSource: IronSourceIdsModel.fromJson(
                  response.data['adIds']['ironSource']),
              facebookIdsModel: FacebookIdsModel.fromJson(
                  response.data['adIds']['facebook'])),
          configModel: ConfigModel(
            isAdsOn: response.data[App.appVersion]['isAdsOn'],
            maxAdFailCounter: response.data[App.appVersion]['maxAdFailCounter'],
            globalClick: response.data[App.appVersion]['globalClick'],
            globalFail: response.data[App.appVersion]['globalFail'],
            screens: response.data[App.appVersion]['screens'],
          ),
        ),
        response.data['adIds'],
        response.data[App.appVersion]['globalClick'],
        response.data[App.appVersion]['globalFail'],
        response.data[App.appVersion]['globalBanner'],
        response.data[App.appVersion]['screens'],
        isAdsOn: response.data[App.appVersion]['isAdsOn'],
        setMaxCount: response.data[App.appVersion]['maxAdFailCounter'],
        isUpdate: response.data[App.appVersion]['isUpdate'],
        updateUrl: response.data[App.appVersion]['updateUrl'],
      );
      App.appBanner =
          App.adPluginX.getAdEngine(context, false).getBanner(context);
      Future.delayed(const Duration(seconds: 1), () {
        App.adPluginX.adNavigatorPushReplacement(context, MainScreen.route);
      });
      // Navigator.pushNamedAndRemoveUntil(context, MainScreen.route, (route) => false);
    }).onError((error, stackTrace) {
      LoggerX.printError("$error");
      LoggerX.printError("$stackTrace");
      LoggerX.printError("Main Json Fetched Failed");
    });
  }
}
