import 'package:adpluginx/AdPlugin/AdEngine.dart';
import 'package:adpluginx/AdPlugin/Google/AdmobFullScreenX.dart';
import 'package:adpluginx/AdPlugin/Unity/UnityFullScreenX.dart';
import 'package:adpluginx/Models/MainModel.dart';
import 'package:adpluginx/utils/LoggerX.dart';
import 'package:facebook_audience_network/facebook_audience_network.dart';
import 'package:flutter/material.dart';
import 'package:ironsource_mediation/ironsource_mediation.dart';
import 'package:provider/provider.dart';

class AdCore {
  static final AdCore _adCore = AdCore._internal();

  factory AdCore() {
    return _adCore;
  }

  AdCore._internal();

  AdCore.init(
    BuildContext context,
    MainModel mainModel,
    Map adsId,
    List globalClick,
    Map globalFails,
    int globalBanner,
    Map screens, {
    bool isAdsOn = false,
    int setMaxCount = 3,
    required bool isUpdate,
    required String updateUrl,
  }) {
    final AdEngine adEngine = Provider.of<AdEngine>(context, listen: false);
    LoggerX.printSuccess("=====> Printing Main Model");
    LoggerX.printSuccess(mainModel.toJson().toString());
    LoggerX.printSuccess("=====> Printing Main Model");
    Future.microtask(() {
      adEngine.setIsAdsOn(mainModel.configModel!.isAdsOn!);
      adEngine.setGlobalClick(mainModel.configModel!.globalClick!);
      adEngine.setGlobalFail(mainModel.configModel!.globalFail!);
      adEngine.setScreensData(mainModel.configModel!.screens!);
      adEngine.setAdsId(mainModel.adIds!.toJson());
      adEngine.createIndexForAdState();
      adEngine.setMaxCount(setMaxCount);
      adEngine.setGlobalBanner(globalBanner);
      adEngine.setIsUpdate(isUpdate);
      adEngine.setUpdateUrl(updateUrl);
      if (!isAdsOn || isUpdate) {
        return;
      }

      adEngine.setInit(true, context);

      /// Init AdMob
      AdmobFullScreenX admobFullScreenX = AdmobFullScreenX();
      admobFullScreenX.initAdMob(context);

      /// Init Unity
      UnityFullScreenX unityFullScreenX = UnityFullScreenX();
      unityFullScreenX.initUnity(adsId['unity']['gameId']);
      // unityFullScreenX.initUnity("4764348");

      /// IronSource Init
      IronSource.init(
        appKey: adsId['ironSource']['appId'],
        adUnits: [
          IronSourceAdUnit.Interstitial,
          IronSourceAdUnit.Banner,
        ],
      );
      // IronSource.init(
      //   appKey: "f15d1fd9",
      //   adUnits: [
      //     IronSourceAdUnit.Interstitial,
      //     IronSourceAdUnit.Banner,
      //   ],
      // );
      IronSource.validateIntegration();
    });

    /// Init FaceBook
    FacebookAudienceNetwork.init(
        testingId: "37b1da9d-b48c-4103-a393-2e095e734bd6", //optional
        iOSAdvertiserTrackingEnabled: true //default false
        );
  }

  static Widget getBanner(BuildContext context) {
    final AdEngine adEngine = Provider.of<AdEngine>(context, listen: false);
    return adEngine.getBanner(context);
  }

  AdCore.callAds(BuildContext context, Function action) {
    final AdEngine adEngine = Provider.of<AdEngine>(context, listen: false);
    adEngine.adActionController(context, () {
      action();
    });
  }

  AdCore.navigator(BuildContext context, String namedRoute, {Map? arguments}) {
    AdCore.callAds(context, () {
      Navigator.pushNamed(context, namedRoute, arguments: arguments);
    });
  }

  AdCore.navigatorPushedReplacement(BuildContext context, String namedRoute,
      {Map? arguments}) {
    AdCore.callAds(context, () {
      Navigator.pushNamedAndRemoveUntil(context, namedRoute, (route) => false,
          arguments: arguments);
    });
  }
}
