import 'package:adpluginx/AdPlugin/FaceBook/FacebookFullScreenX.dart';
import 'package:adpluginx/AdPlugin/Google/AdmobBannerX.dart';
import 'package:adpluginx/AdPlugin/Google/AdmobFullScreenX.dart';
import 'package:adpluginx/AdPlugin/IronSource/IronSourceBannerX.dart';
import 'package:adpluginx/AdPlugin/IronSource/IronSourceFullScreenX.dart';
import 'package:adpluginx/AdPlugin/Unity/UnityFullScreenX.dart';
import 'package:adpluginx/Models/Ads/Facebook/FacebookIdsModel.dart';
import 'package:adpluginx/Models/Ads/Google/GoogleIdsModel.dart';
import 'package:adpluginx/Models/Ads/Unity/UnityIdsModel.dart';
import 'package:adpluginx/Models/MainModel.dart';
import 'package:adpluginx/Widgets/g_native.dart';
import 'package:adpluginx/utils/LoggerX.dart';
import 'package:flutter/material.dart';
import 'package:ironsource_mediation/ironsource_mediation.dart';

class AdEngine extends ChangeNotifier {
  /// From Json Variables
  bool isAdsOn = false;
  String updateLink = "https://www.app.store.com";
  bool isUpdate = false;
  List globalClick = [];
  Map globalFail = {};
  Map screens = {};
  Map adsId = {};
  int maxFailCount = 3;
  int globalBanner = 0;
  Widget banner = const Text("Init");
  final ValueNotifier<bool> isInit = ValueNotifier<bool>(false);

  MainModel? mainModel;

  /// Local State Variables
  int failCounter = 0;
  Map index = {
    "global": 0,
  };
  bool isLoading = false;

  bool clickOnce = false;

  setInit(bool data, BuildContext context) {
    isInit.value = data;
    LoggerX.printInformation("Calling Init Change=============> Data");
    notifyListeners();
    isInit.addListener(() {
      LoggerX.printInformation("Value Changed=============> ${isInit.value}");
      banner = getBanner(context);
      notifyListeners();
    });
    isInit.value = data;
    notifyListeners();
  }

  createIndexForAdState() {
    assert(screens.isNotEmpty);
    for (int i = 0; i < screens.keys.length; i++) {
      index[screens.keys.toList()[i]] = 0;
    }
    notifyListeners();
    LoggerX.printSuccess("Created Indexation ====> $index");
  }

  setMaxCount(int? data) {
    maxFailCount = data ?? 3;
    LoggerX.printSuccess("Set completed ad flags ====> $data");
    notifyListeners();
  }

  setIsAdsOn(bool data) {
    isAdsOn = data;
    LoggerX.printSuccess("Set completed ad flags ====> $data");
    notifyListeners();
  }

  setUpdateUrl(String data) {
    updateLink = data;
    LoggerX.printSuccess("Set Update Url ====> $data");
    notifyListeners();
  }

  setIsUpdate(bool data) {
    isUpdate = data;
    LoggerX.printSuccess("Set IsUpdate flags ====> $data");
    notifyListeners();
  }

  setGlobalBanner(int data) {
    globalBanner = data;
    LoggerX.printSuccess("Set completed GlobalBanner ====> $data");
    notifyListeners();
  }

  getNative({double? height}) {
    if (adsId.isEmpty) {
      return const SizedBox();
    }
    return GNative(
      id: (adsId['google'] as GoogleIdsModel).native!,
      isAdOn: isAdsOn,
      height: height,
    );
  }

  setGlobalClick(List? data) {
    assert(data != null);
    assert(data!.isNotEmpty);
    globalClick = data!;
    LoggerX.printSuccess("Set completed global clicks list ====> $data");
    notifyListeners();
  }

  setGlobalFail(Map? data) {
    assert(data != null);
    assert(data!.isNotEmpty);
    globalFail = data!;
    LoggerX.printSuccess("Set completed global fails list ====> $data");
    notifyListeners();
  }

  setScreensData(Map? data) {
    assert(data != null);
    assert(data!.isNotEmpty);
    screens = data!;
    LoggerX.printSuccess("Set completed screen wise sequence ====> $data");
    notifyListeners();
  }

  setAdsId(Map? data) {
    assert(data != null);
    assert(data!.isNotEmpty);
    adsId = data!;
    LoggerX.printSuccess("Set completed advertisement ids ====> $data");
    notifyListeners();
  }

  setClickOnceOn() {
    LoggerX.printSuccess("Setting Click On");
    clickOnce = true;
    notifyListeners();
  }

  setClickOnceOff() {
    LoggerX.printSuccess("Setting Click Off");
    clickOnce = false;
    notifyListeners();
  }

  adActionController(BuildContext context, Function action) {
    setClickOnceOn();
    ModalRoute? route = ModalRoute.of(context);

    Map? fetchedScreen = screens[route?.settings.name];
    LoggerX.printInformation(
        "Logging Current Route =====> ${route!.settings.name}");
    LoggerX.printInformation(
        "Logging Fetch Screen Route =====> $fetchedScreen");
    if (route != null && fetchedScreen != null) {
      /// If Route Exist
      LoggerX.printInformation("Logging Local Screen =====> $fetchedScreen");
      LoggerX.printInformation(
          "Logging Current Route =====> ${route.settings.name}");

      bool isAdOnLocal = fetchedScreen['localAdFlag'] ?? isAdsOn;

      /// Checking If Local Ad Flag is On
      if (!isAdOnLocal) {
        action();
        setClickOnceOff();
        return;
      }
      List localClick = fetchedScreen['localClick'] ?? [];
      // List localFail = fetchedScreen['localFail'] ?? [];
      int localIndex = index[route.settings.name] ?? 0;
      LoggerX.printInformation("Logging Local Index =====> $localIndex");
      adHandler(context, () {
        action();
        setClickOnceOff();
        incrementLocalCLickIndex(route.settings.name!);
      }, localClick[localIndex]);
    } else {
      /// Take GlobalClicker
      int globalIndex = index["global"] ?? 0;
      LoggerX.printInformation("Logging Global Index =====> $globalIndex");
      adHandler(context, () {
        action();
        setClickOnceOff();
        incrementGlobalCLickIndex();
      }, globalClick[globalIndex]);
    }
  }

  Widget getBanner(BuildContext context) {
    ModalRoute? route = ModalRoute.of(context);

    Map? fetchedScreen = screens[route?.settings.name];
    LoggerX.printInformation(
        "Logging Current Route =====> ${route!.settings.name}");
    LoggerX.printInformation(
        "Logging Fetch Screen Route =====> $fetchedScreen");
    if (route != null && fetchedScreen != null) {
      LoggerX.printInformation("Logging Local Screen =====> $fetchedScreen");
      LoggerX.printInformation(
          "Logging Current Route =====> ${route.settings.name}");

      bool isAdOnLocal = fetchedScreen['localAdFlag'] ?? isAdsOn;
      if (!isAdOnLocal) {
        return const SizedBox(
          height: 0,
          width: 0,
        );
      }
      int localBannerIndex = fetchedScreen['banner'] ?? globalBanner;
      return showBanner(localBannerIndex);
    } else {
      /// Take GlobalClicker
      return showBanner(globalBanner);
    }
  }

  Widget showBanner(int index) {
    if (adsId.isEmpty) {
      return const SizedBox();
    }
    switch (index) {
      case 0:
        return AdmobBannerX(
          id: (adsId['google'] as GoogleIdsModel).banner!,
        );
      case 1:
        return const IronSourceBannerX();
      default:
        return const SizedBox(
          width: 0,
          height: 0,
        );
    }
  }

  incrementGlobalCLickIndex() {
    int globalIndex = index["global"] ?? 0;
    LoggerX.printInformation(
        "Global Click Length =====> ${globalClick.length}");
    LoggerX.printInformation("Global Index Length =====> $globalIndex");

    if (globalClick.length > (globalIndex + 1)) {
      /// Increment Index
      LoggerX.printInformation(
          "Before Incrementing Local Index =====> ${index["global"]}");
      index["global"] = globalIndex + 1;
      notifyListeners();
      LoggerX.printInformation(
          "After Incrementing Local Index =====> ${index["global"]}");
    } else {
      /// Reset Index
      LoggerX.printInformation("Reset Local Index =====> ${index["global"]}");
      index["global"] = 0;
      notifyListeners();
    }
  }

  incrementLocalCLickIndex(String routeName) {
    Map? fetchedScreen = screens[routeName];
    if (fetchedScreen == null) {
      LoggerX.printError("$routeName is Not Available from Server Side");
      return;
    }
    int localIndex = index[routeName] ?? 0;
    List localClick = fetchedScreen['localClick'] ?? [];

    LoggerX.printInformation("Local Click Length =====> ${localClick.length}");
    LoggerX.printInformation("Local Index Length =====> $localIndex");

    if (localClick.length > (localIndex + 1)) {
      /// Increment Index
      LoggerX.printInformation(
          "Before Incrementing Local Index =====> ${index[routeName]}");
      index[routeName] = localIndex + 1;
      notifyListeners();
      LoggerX.printInformation(
          "After Incrementing Local Index =====> ${index[routeName]}");
    } else {
      /// Reset Index
      LoggerX.printInformation("Reset Local Index =====> ${index[routeName]}");
      index[routeName] = 0;
      notifyListeners();
    }
  }

  /// ResetFail Counter
  resetFailCounter() {
    failCounter = 0;
    notifyListeners();
  }

  adHandler(BuildContext context, Function? action, int provider,
      {bool fromFail = false}) async {
    if (!isAdsOn) {
      action!();
      return;
    }

    /// Fail Counter Handling
    if (fromFail) {
      if (failCounter < maxFailCount) {
        failCounter++;
      } else {
        action!();
        offLoader();
        resetFailCounter();
      }
      notifyListeners();
    }

    /// Ad Cases Handling

    switch (provider) {
      case 0:

        /// Calling Admob Full Screen Ad Loader
        LoggerX.printInformation("------Showing Google Ads-----");
        onLoader();
        AdmobFullScreenX admobFullScreenX = AdmobFullScreenX();
        admobFullScreenX.showGoogleFullScreenAds(
            (adsId['google'] as GoogleIdsModel).fullScreen!, onComplete: () {
          resetFailCounter();
          action!();
          offLoader();
        }, onFailed: () {
          LoggerX.printError("-----Failed on Google ------------");
          Future.microtask(() => onFailedAds(context, 0, action!));
        });
        break;
      case 1:
        LoggerX.printInformation("------Showing Unity Ads-----");
        onLoader();
        UnityFullScreenX unityFullScreenX = UnityFullScreenX();
        unityFullScreenX.showUnityAds(
            (adsId['unity'] as UnityIdsModel).placementId, onComplete: () {
          resetFailCounter();
          action!();
          offLoader();
        }, onFailed: () {
          LoggerX.printError("-----Failed on Unity ------------");
          Future.microtask(() => onFailedAds(context, 1, action!));
        });
        break;
      case 2:
        LoggerX.printInformation("------Showing IronSource Ads-----");
        onLoader();
        IronSourceFullScreenX ironSourceFullScreenX =
            IronSourceFullScreenX.onInit(
          onClose: () {
            resetFailCounter();
            action!();
            offLoader();
          },
          onFailed: () {
            LoggerX.printError("----- Failed on IronSource ------------");
            Future.microtask(() => onFailedAds(context, 2, action!));
          },
          onLoaded: () {
            IronSource.showInterstitial();
          },
        );
        IronSource.setISListener(ironSourceFullScreenX);
        IronSource.loadInterstitial();
        break;
      case 3:
        LoggerX.printInformation("------Showing Facebook Ads-----");
        onLoader();
        FacebookFullScreenX facebookFullScreenX = FacebookFullScreenX();
        facebookFullScreenX.showFaceBookFullScreenAds(
            (adsId['facebook'] as FacebookIdsModel).fullScreen!, onClosed: () {
          resetFailCounter();
          action!();
          offLoader();
        }, onFailed: () {
          LoggerX.printError("-----Failed on Facebook ------------");
          Future.microtask(() => onFailedAds(context, 3, action!));
        });
        break;

      default:
        resetFailCounter();
        action!();
        offLoader();
    }
  }

  /// This will handle fail cases
  onFailedAds(BuildContext context, int adFailedFrom, Function action) {
    var route = ModalRoute.of(context);
    String? routeName = route?.settings.name;
    Map? fetchFailedIndex = screens[routeName];
    if (routeName != null && fetchFailedIndex != null) {
      int? localFailed = fetchFailedIndex['localFail'][adFailedFrom.toString()];
      if (localFailed != null) {
        /// Show Fetch Local Failed
        LoggerX.printInformation("Local Failure Index =======> $localFailed");
        adHandler(context, action, localFailed);
      } else {
        /// Show Global Failed
        int? globalFailed = globalFail[adFailedFrom.toString()];
        assert(globalFailed != null);
        LoggerX.printInformation("Local Failure Index =======> $localFailed");
        adHandler(context, action, globalFailed!);
      }
    } else {
      /// Show Global Failed
      int? globalFailed = globalFail[adFailedFrom.toString()];
      assert(globalFailed != null);
      adHandler(context, action, globalFailed!);
    }
  }

  onLoader() {
    isLoading = true;
    notifyListeners();
  }

  offLoader() {
    isLoading = false;
    notifyListeners();
  }

  /// For Test Purpose Only
  Future<bool> adCallingFuture({isError = false}) async {
    return Future.delayed(const Duration(milliseconds: 700), () => !isError);
  }
}
