import 'package:flutter/material.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';

class GNative extends StatefulWidget {
  final double? height;
  final String id;
  final bool isAdOn;

  const GNative({Key? key, this.height, required this.id, required this.isAdOn})
      : super(key: key);

  @override
  State<GNative> createState() => _GNativeState();
}

class _GNativeState extends State<GNative> {
  bool isLoading = true;
  bool isFailed = false;
  bool facebookError = false;
  NativeAd? native;
  AdWidget? adWidget;
  double? height;
  NativeAdListener? listener;

  @override
  void initState() {
    listener = NativeAdListener(
      // Called when an ad is successfully received.
      onAdLoaded: (Ad ad) {
        if (mounted) {
          setState(() {
            isLoading = false;
          });
        }
      },
      // Called when an ad request failed.
      onAdFailedToLoad: (Ad ad, LoadAdError error) {
        print(error.message);
        if (mounted) {
          setState(() {
            isLoading = false;
            isFailed = true;
          });
        }
        ad.dispose();
      },
    );

    if (widget.isAdOn) {
      native = NativeAd(
        adUnitId: widget.id,
        factoryId: 'adFactoryExample',
        nativeAdOptions: NativeAdOptions(
          shouldRequestMultipleImages: true,
          adChoicesPlacement: AdChoicesPlacement.topLeftCorner,
        ),
        request: const AdRequest(),
        listener: listener!,
      );
      native!.load();
      adWidget = AdWidget(ad: native!);
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return isLoading || facebookError
        ? const SizedBox(
            height: 0,
            width: 0,
          )
        : isFailed
            ? const SizedBox(
                height: 0,
                width: 0,
              )
            : Container(
                margin: const EdgeInsets.all(10.0),
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(10.0),
                  color: const Color(0xFF343434),
                ),
                alignment: Alignment.center,
                width: double.infinity,
                height: 270,
                child: ClipRRect(
                  borderRadius: BorderRadius.circular(10.0),
                  child: adWidget!,
                ),
              );
  }
}
